package analyzer

import "testing"

func TestFunctionUniqueErr (test *testing.T) {
testStringErr (test,
"hello already declared at stream0.fspl:2:1", 3, 1,
`
[hello] = { }
[hello] = { }
`)
}

func TestFunctionUniqueErrShadowBuiltin (test *testing.T) {
testStringErr (test,
"cannot shadow builtin true ", 2, 2,
`
[true]:Bool = false
`)
}

func TestFunctionUnique (test *testing.T) {
testString (test,
`
[hello] = { }
[world] = { }
`)
}

func TestTypeUniqueErr (test *testing.T) {
testStringErr (test,
"hello already declared at stream0.fspl:2:1", 3, 1,
`
hello: *Int
hello: (x:Int y:Int)
`)
}

func TestTypeUniqueErrShadowBuiltin (test *testing.T) {
testStringErr (test,
"cannot shadow builtin true ", 2, 2,
`
true:Int
`)
}

func TestTypeUnique (test *testing.T) {
testString (test,
`
hello: *Int
world: (x:Int y:Int)
`)
}

func TestTypeNameErrMissing (test *testing.T) {
testStringErr (test,
"no type named example", 2, 20,
`
[main] = { example:Example }
`)
}

func TestTypeName (test *testing.T) {
testString (test,
`
Example: Int
AllBuiltin: (
	int:    Int
	uint:   UInt
	byte:   Byte
	rune:   Rune
	string: String
	i8:     I8
	i8:     I16
	i8:     I32
	i8:     I64
	u8:     U8
	u8:     U16
	u8:     U32
	u8:     U64
	f32:    F32
	f64:    F64
	bool:   Bool)
[main] = {
	example:Example
	allBuiltin:AllBuiltin
}
`)
}

func TestFunctionUniqueErr (test *testing.T) {
testStringErr (test,
"hello already declared at stream0.fspl:2:1", 3, 1,
`
hello: *Int
[hello] = { }
`)
}

func TestFunctionUnique (test *testing.T) {
testString (test,
`
hello: *Int
[world] = { }
`)
}

func TestMethodUniqueErr (test *testing.T) {
testStringErr (test,
"Bird.fly already declared at stream0.fspl:3:1", 4, 1,
`
Bird: Int
Bird::[fly] = { }
Bird::[fly distance:Int] = { }
`)
}

func TestMethodUnique (test *testing.T) {
testString (test,
`
Bird: Int
Bird::[fly] = { }
Bird::[land] = { }
Bird::[walk distance:Int] = { }
Bat: Int
Bat::[fly] = { }
[fly] = { }
`)
}

func TestMethodNameErrMissing (test *testing.T) {
testStringErr (test,
"no method Type::world", 5, 3,
`
Type: Int
Type::[something] = { }
[main] = {
	instance:Type
	[instance::world]
}
`)
}

func TestMethodName (test *testing.T) {
testString (test,
`
Type: Int
Type::[world] = { }
[main] = {
	instance:Type
	[instance::world]
}
`)
}

func TestBehaviorUniqueErr (test *testing.T) {
testStringErr (test,
"fly already listed in interface at stream0.fspl:2:8", 2, 14,
`
Bird: ([fly] [fly])
`)
}

func TestBehaviorUnique (test *testing.T) {
testString (test,
`
Bird: ([fly] [land])
`)
}

func TestBehaviorNameErrMissing (test *testing.T) {
testStringErr (test,
"no behavior Bird::swim", 8, 3,
`
Bird: ([fly] [land])
BirdImpl: Int
BirdImpl::[fly] = { }
BirdImpl::[land] = { }
[main] = {
	bird:Bird = [@impl:BirdImpl]
	[bird::swim]
}
`)
}

func TestBehaviorName (test *testing.T) {
testString (test,
`
Bird: ([fly] [land])
BirdImpl: Int
BirdImpl::[fly] = { }
BirdImpl::[land] = { }
[main] = {
	bird:Bird = [@impl:BirdImpl]
	[bird::fly]
}
`)
}

func TestFunctionArgumentUniqueErr (test *testing.T) {
testStringErr (test,
"x already declared in block at stream0.fspl:2:8", 2, 14,
`
[main x:Int x:U8 y:Int] = { }
`)
}

func TestFunctionArgumentUnique (test *testing.T) {
testString (test,
`
[main x:Int y:U8 z:Int] = { }
`)
}

func TestMethodArgumentUniqueErr (test *testing.T) {
testStringErr (test,
"x already declared in block at stream0.fspl:3:13", 3, 19,
`
Bird: Int
Bird::[main x:Int x:U8 y:Int] = { }
`)
}

func TestMethodArgumentUnique (test *testing.T) {
testString (test,
`
Bird: Int
Bird::[main x:Int y:U8 z:Int] = { }
`)
}

func TestVariableUniqueErr (test *testing.T) {
testStringErr (test,
"x already declared in block at stream0.fspl:3:2", 5, 2,
`
[main] = {
	x:Int
	{ x:U8 }
	x:U8
}
`)
}

func TestVariableUniqueErrShadowBuiltin (test *testing.T) {
testStringErr (test,
"cannot shadow builtin false ", 3, 2,
`
[main] = {
	false:Bool = true
}
`)
}

func TestVariableUnique (test *testing.T) {
testString (test,
`
x: Int
[y] = { }
[main x:Int] = {
	x:Int
	y:Int
	{
		x:U8
		y:U8
	}
	z:Int
}
`)
}

func TestVariableNameErrType (test *testing.T) {
testStringErr (test,
"named type example cannot be used as location expression", 4, 2,
`
example: Int
[main] = {
	example = 5
}
`)
}

func TestVariableNameErrFunction (test *testing.T) {
testStringErr (test,
"function example cannot be used as location expression", 4, 2,
`
[example] = { }
[main] = {
	example = 5
}
`)
}

func TestVariableNameErrMissing (test *testing.T) {
testStringErr (test,
"no variable named example", 4, 2,
`
[main] = {
	example = 5
}
`)
}

func TestVariableNameErrNotInScope (test *testing.T) {
testStringErr (test,
"no variable named example", 6, 2,
`
[main] = {
	{
		example:Int
	}
	example = 5
}
`)
}

func TestVariableName (test *testing.T) {
testString (test,
`
[main] = {
	example:Int
	example = 5
	{
		example = 2
		x:Int
		x = 5
		{
			x = 2
			example = 3
		}
	}

}
`)
}

func TestStructMemberUniqueErr (test *testing.T) {
testStringErr (test,
"x already listed in struct at stream0.fspl:3:2", 6, 2,
`
Bird: (
	x:Int
	y:Int
	z:Int
	x:U8)
`)
}

func TestStructMemberUnique (test *testing.T) {
testString (test,
`
Bird: (
	x:Int
	y:Int
	z:Int
	a:U8)
`)
}

func TestStructMemberNameErrMissing (test *testing.T) {
testStringErr (test,
"no member Type.world", 5, 2,
`
Type: (something:Int)
[main] = {
	instance:Type
	instance.world = 5
}
`)
}

func TestStructMemberName (test *testing.T) {
testString (test,
`
Type: (world:Int)
[main] = {
	instance:Type
	instance.world = 5
}
`)
}

func TestStructLiteralMemberUniqueErr (test *testing.T) {
testStringErr (test,
"x already listed in struct literal at stream0.fspl:5:2", 7, 2,
`
Point: (x:Int y:Int z:Int)
[main] = {
	point:Point = (
		x: 5
		y: 0
		x: 32)
}
`)
}

func TestStructLiteralMemberUnique (test *testing.T) {
testString (test,
`
Point: (x:Int y:Int z:Int)
[main] = {
	point:Point = (
		x: 5
		y: 0
		z: 32)
}
`)
}

func TestFunctionNameErrType (test *testing.T) {
testStringErr (test,
"cannot call named type example", 3, 3,
`
example: Int
[main] = [example]
`)
}

func TestFunctionNameErrVar (test *testing.T) {
testStringErr (test,
"cannot call example:Int", 4, 3,
`
[main] = {
	example:Int
	[example]
}
`)
}

func TestFunctionNameErrMissing (test *testing.T) {
testStringErr (test,
"no function named example", 2, 3,
`
[main] = [example]
`)
}

func TestFunctionName (test *testing.T) {
testString (test,
`
[example] = { }
[main] = [example]
`)
}
