package analyzer

import "testing"

func TestAssignLiteralErrUnsignedNegative (test *testing.T) {
testStringErr (test,
"expected unsigned integer", 3, 11,
`
[main] = {
	x:UInt = -5
}
`)
}

func TestAssignLiteralErrIntegerFloat (test *testing.T) {
testStringErr (test,
"expected integer", 3, 10,
`
[main] = {
	x:Int = 5.5
}
`)
}

func TestAssignLiteralErrIntegerStruct (test *testing.T) {
testStringErr (test,
"expected integer", 3, 10,
`
[main] = {
	x:Int = (x: 5)
}
`)
}

func TestAssignLiteralErrIntegerArray (test *testing.T) {
testStringErr (test,
"expected integer", 3, 10,
`
[main] = {
	x:Int = (* 5)
}
`)
}

func TestAssignLiteralErrIntegerOverflow (test *testing.T) {
testStringErr (test,
"expected smaller number", 3, 11,
`
[main] = {
	x:Byte = 1000
}
`)
}

func TestAssignLiteralErrArrayInt (test *testing.T) {
testStringErr (test,
"expected array", 3, 11,
`
[main] = {
	x:3:Int = 5
}
`)
}

func TestAssignLiteralErrArrayWrongLength (test *testing.T) {
testStringErr (test,
"expected 3 elements", 3, 11,
`
[main] = {
	x:3:Int = (* 1 2 3 4)
}
`)
}

func TestAssignLiteralErrArrayWrongType (test *testing.T) {
testStringErr (test,
"expected integer", 3, 19,
`
[main] = {
	x:3:Int = (* 1 2 3.3 4)
}
`)
}

func TestAssignLiteralErrSliceWrongType (test *testing.T) {
testStringErr (test,
"expected integer", 3, 19,
`
[main] = {
	x:*:Int = (* 1 2 3.3 4)
}
`)
}

func TestAssignLiteralErrStructWrongType (test *testing.T) {
testStringErr (test,
"expected integer", 3, 26,
`
[main] = {
	x:3:(x:Int y:Int) = (x: 5.5 y: 3)
}
`)
}
func TestAssignLiteralErrStructInteger (test *testing.T) {
testStringErr (test,
"expected struct", 3, 26,
`
[main] = {
	x:3:(x:Int y:Int) = 5
}
`)
}

func TestAssignLiteral (test *testing.T) {
testString (test,
`
[main] = {
	a:F64 = 5.3
	b:F32 = 5.3
	c:F64 = -5
	d:F32 = -5
	e:Byte = 64
	z:UInt = 5
	x:Int  = -5
	arr:4:2:Int = (*
		(* 1 2)
		(* 3 4)
		(* 5 6)
		(* 7 8))
	slice:*:Int = (* 3 1 2 3)
	struct:(x:Int y:Int) = (
		x: 9
		y: 10)
}
`)
}
